/*  OctoWS2811 BasicTest.ino - Basic RGB LED Test
    http://www.pjrc.com/teensy/td_libs_OctoWS2811.html
    Copyright (c) 2013 Paul Stoffregen, PJRC.COM, LLC

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
    THE SOFTWARE.

  Required Connections
  --------------------
    pin 2:  LED Strip #1    OctoWS2811 drives 8 LED Strips.
    pin 14: LED strip #2    All 8 are the same length.
    pin 7:  LED strip #3
    pin 8:  LED strip #4    A 100 ohm resistor should used
    pin 6:  LED strip #5    between each Teensy pin and the
    pin 20: LED strip #6    wire to the LED strip, to minimize
    pin 21: LED strip #7    high frequency ringining & noise.
    pin 5:  LED strip #8
    pin 15 & 16 - Connect together, but do not use
    pin 4 - Do not use
    pin 3 - Do not use as PWM.  Normal use is ok.

  This test is useful for checking if your LED strips work, and which
  color config (WS2811_RGB, WS2811_GRB, etc) they require.
*/

#include <OctoWS2811.h>

const int LED_COUNT = 180;
const int LED_BUFFER_SIZE = 240;

DMAMEM int displayMemory[LED_BUFFER_SIZE*6];
int drawingMemory[LED_BUFFER_SIZE*6];

const int config = WS2811_RGB | WS2811_800kHz;

OctoWS2811 leds(LED_BUFFER_SIZE, displayMemory, drawingMemory, config);

int phase = 0;

void setup() {
  leds.begin();

  for (int i=0; i<LED_BUFFER_SIZE; i++) {
    leds.setPixel(i, 0x0);
  }

  leds.show();
}

#define RED    0x0F0000
#define GREEN  0x000F00
#define GREEN2 0x081F08
#define BLUE   0x00000F
#define BLUE2  0x08081F
#define YELLOW 0x0F0F08
#define PINK   0x0F0408
#define ORANGE 0x0F0402
#define WHITE  0x0f0f0f

void loop() {
  int microsec = 2000000 / LED_COUNT;  // change them all in 2 seconds

  // uncomment for voltage controlled speed
  // millisec = analogRead(A9) / 40;

  ohGod();

  /*
  colorWipe(RED, microsec);
  colorWipe(GREEN, microsec);
  colorWipe(GREEN2, microsec);
  colorWipe(BLUE, microsec);
  colorWipe(BLUE2, microsec);
  colorWipe(YELLOW, microsec);
  colorWipe(PINK, microsec);
  colorWipe(ORANGE, microsec);
  colorWipe(WHITE, microsec);
  */
}

void ohGod()
{
  for (int i=0; i<LED_COUNT; i++) {
    //uint_fast8_t br = random(0,0xff);

    uint_fast16_t p = (phase+i*13) & 0xff;

    // FIXME
    //uint_fast16_t p = ((uint_fast16_t)(sin(phase+i*0.01) * 0xff)) & 0xff;

    if( p > 0x7f ) p = 0xff - p;
    p <<= 1;  // 0..0xff
    p = (p*p)>>8; // lazy sorta-gamma correction

    uint_fast8_t b = min( 0xff, p<<1 );
    uint_fast8_t g = max( 0, (int_fast16_t)(p<<1)-0xff );
    uint_fast8_t w = max( 0, (int_fast16_t)(p<<2)-0x3ff );

    int color = ((w<<16) | (g<<8) | b);

    if( i < LED_COUNT/2 ) {
      setRGBW(i, color);
    } else {
      leds.setPixel((i*4)/3, color);
    }
  }
  leds.show();

  phase++;

  delay(10);
}

void colorWipe(int color, int wait)
{
  for (int i=0; i < LED_COUNT; i++) {
    //leds.setPixel(i, color);
    setRGBW(i, color);
    leds.show();
    delayMicroseconds(wait);
  }
}

void setRGBW(int idx, int color)
{
  uint_fast8_t r = ((color&0xff0000)>>16);
  uint_fast8_t g = ((color&0x00ff00)>>8);
  uint_fast8_t b =  (color&0x0000ff);

  uint_fast8_t w = min( min( r, g ), b );

  // Set the colorful RGB part
  int colorIdx = (idx * 4) / 3;
  int colorMinusWhite = ((r-w)<<16) | ((g-w)<<8) | (b-w);
  colorMinusWhite = color;  // FIXME
  leds.setPixel(colorIdx, colorMinusWhite);

  // Set the white
  int whiteIdx = colorIdx/4;  // round to nearest group of 3 LEDs
  whiteIdx = whiteIdx * 4 + 3;  // target the final 3 color components. Chipset order is: RGB,RGB,RGB,WWW

  int oldW = leds.getPixel(whiteIdx);
  int newW = 0;

  // Holy cats, the W component is bright
  //w = random(0,0xff);

  switch( colorIdx % 4 ) {
    case 0:  newW = (oldW&0x00ffff) | (w<<16); break;
    case 1:  newW = (oldW&0xff00ff) | (w<<8); break;
    case 2:  newW = (oldW&0xffff00) | w; break;
  }

  leds.setPixel( whiteIdx, newW );
}
